@@ -121,6 +121,19 @@ notification_plugin_configure (XfcePanelPlugin      *plugin,
   xfconf_g_property_bind (notification_plugin->channel, SETTING_LOG_ONLY_TODAY, G_TYPE_BOOLEAN,
                           G_OBJECT (check), "active");
 
+  label = gtk_label_new (_("Hide 'Clear log' confirmation dialog"));
+#if GTK_CHECK_VERSION (3, 16, 0)
+  gtk_label_set_xalign (GTK_LABEL (label), 0);
+#else
+  gtk_widget_set_halign (label, GTK_ALIGN_START);
+#endif
+  gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (label), 0, 2, 1, 1);
+  check = gtk_switch_new ();
+  gtk_widget_set_halign (check, GTK_ALIGN_END);
+  gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (check), 1, 2, 1, 1);
+  xfconf_g_property_bind (notification_plugin->channel, SETTING_HIDE_CLEAR_PROMPT, G_TYPE_BOOLEAN,
+                          G_OBJECT (check), "active");
+
   /* link the dialog to the plugin, so we can destroy it when the plugin
    * is closed, but the dialog is still open */
   g_object_set_data (G_OBJECT (plugin), "dialog", dialog);
