@@ -27,7 +27,7 @@
 #include <gtk/gtk.h>
 #include <xfconf/xfconf.h>
 #include <libxfce4util/libxfce4util.h>
-#include <libxfce4panel/xfce-panel-plugin.h>
+#include <libxfce4panel/libxfce4panel.h>
 
 #include "notification-plugin.h"
 #include "notification-plugin-log.h"
@@ -88,12 +88,20 @@ cb_button_pressed (GtkButton *button,
                    GdkEventButton *event,
                    NotificationPlugin *notification_plugin)
 {
-  if (event->button != 1)
-    return FALSE;
-  if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
-    notification_plugin_popup_menu (notification_plugin);
+  if (event->button == 1 && !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
+    {
+      notification_plugin_popup_menu (notification_plugin);
+      return TRUE;
+    }
 
-  return TRUE;
+  if (event->button == 2)
+    {
+      gboolean state = xfconf_channel_get_bool (notification_plugin->channel, "/do-not-disturb", FALSE);
+      xfconf_channel_set_bool (notification_plugin->channel, "/do-not-disturb", !state);
+      return TRUE;
+    }
+
+  return FALSE;
 }
 
 
@@ -201,7 +209,7 @@ notification_plugin_new (XfcePanelPlugin *panel_plugin)
   gboolean               state;
 
   /* Allocate memory for the plugin structure */
-  notification_plugin = panel_slice_new0 (NotificationPlugin);
+  notification_plugin = g_slice_new0 (NotificationPlugin);
   notification_plugin->plugin = panel_plugin;
 
   /* Initialize xfconf */
@@ -271,7 +279,7 @@ notification_plugin_free (XfcePanelPlugin *plugin,
     g_source_remove (notification_plugin->menu_size_allocate_next_handler);
 
   /* free the plugin structure */
-  panel_slice_free (NotificationPlugin, notification_plugin);
+  g_slice_free (NotificationPlugin, notification_plugin);
 }
 
 
